unit BrowseDir;

interface

uses
  Windows, SysUtils, Controls, Classes, Messages, StdCtrls, ExtCtrls,
  SelectDirectoryDialog, ASButton;

type
  TBrowseDirPosition1 = (bdpAbove, bdpLeft);
  TBrowseDirPosition2 = (bdpBelow, bdpRight);
  TOnBeforeShowDialog = function(Sender: TObject):boolean of object;
  TbdLabel = class(TLabel)
  public
    property OnResize;
  end;
  TbdButton = class(TASButton)
  public
    property OnResize;
  end;
  TCustomBrowseDir = class(TCustomPanel)
  private
    FSubLabel: TbdLabel;
    FSubLabelPosition: TBrowseDirPosition1;
    FSubLabelSpacing: Integer;
    FSubEdit: TEdit;
    FSubButton: TbdButton;
    FSubButtonPosition: TBrowseDirPosition2;
    FSubButtonSpacing: Integer;
    FLabelPanel: TPanel;
    FEditPanel: TPanel;
    FButtonPanel: TPanel;
    FSubDialog: TSelectDirectoryDialog;
    FOnBeforeShowDialog: TOnBeforeShowDialog;
    procedure SetLabelPosition(const Value: TBrowseDirPosition1);
    procedure SetLabelSpacing(const Value: Integer);
    procedure SetButtonPosition(const Value: TBrowseDirPosition2);
    procedure SetButtonSpacing(const Value: Integer);
  protected
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    procedure CMEnabledchanged(var Message: TMessage); message CM_ENABLEDCHANGED;
    procedure CMBidimodechanged(var Message: TMessage); message CM_BIDIMODECHANGED;
    procedure SubLabelResize(Sender: TObject);
    procedure SubButtonResize(Sender: TObject);
    procedure SubButtonClick(Sender: TObject);
  public
    constructor Create(AOwner: TComponent); override;
    property SubLabel: TbdLabel read FSubLabel;
    property SubLabelPosition: TBrowseDirPosition1 read FSubLabelPosition write SetLabelPosition;
    property SubLabelSpacing: Integer read FSubLabelSpacing write SetLabelSpacing;
    property SubEdit: TEdit read FSubEdit;
    property SubButton: TbdButton read FSubButton;
    property SubButtonPosition: TBrowseDirPosition2 read FSubButtonPosition write SetButtonPosition;
    property SubButtonSpacing: Integer read FSubButtonSpacing write SetButtonSpacing;
    property SubDialog: TSelectDirectoryDialog read FSubDialog;
    property OnBeforeShowDialog: TOnBeforeShowDialog read FOnBeforeShowDialog write FOnBeforeShowDialog;
  end;
  TBrowseDir = class(TCustomBrowseDir)
  public
    property DockManager;
  published
    property Align;
    property Alignment;
    property Anchors;
    property AutoSize;
    property BevelInner;
    property BevelOuter;
    property BevelWidth;
    property BiDiMode;
    property BorderWidth;
    property BorderStyle;
    property Color;
    property Constraints;
    property Ctl3D;
    property UseDockManager default True;
    property DockSite;
    property DragCursor;
    property DragKind;
    property DragMode;
    property Enabled;
    property FullRepaint;
    property Font;
    property Locked;
    property ParentBiDiMode;
    property ParentBackground;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property SubLabel;
    property SubLabelPosition default bdpAbove;
    property SubLabelSpacing default 3;
    property SubEdit;
    property SubButton;
    property SubButtonPosition default bdpRight;
    property SubButtonSpacing default 3;
    property SubDialog;
    property TabOrder;
    property TabStop;
    property Visible;
    property OnBeforeShowDialog;
    property OnCanResize;
    property OnClick;
    property OnConstrainedResize;
    property OnContextPopup;
    property OnDockDrop;
    property OnDockOver;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDock;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnGetSiteInfo;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnResize;
    property OnStartDock;
    property OnStartDrag;
    property OnUnDock;
  end;

procedure Register;

implementation

{$R *.res}

constructor TCustomBrowseDir.Create(AOwner: TComponent);
  const
    SubDialogInitialDir = '.';
  begin
    inherited Create(AOwner);
    Parent:=TWinControl(AOwner);
    Caption:='';
    BevelOuter:=bvNone;
    if not Assigned(FLabelPanel) then begin
      FLabelPanel:=TPanel.Create(Self);
      FLabelPanel.Parent:=Self;
      FLabelPanel.FreeNotification(Self);
      FLabelPanel.Name:='LabelPanel';
      FLabelPanel.SetSubComponent(True);
      FLabelPanel.Caption:='';
      FLabelPanel.ParentBackground:=True;
      FLabelPanel.ParentColor:=True;
      FLabelPanel.BevelOuter:=bvNone;
      FLabelPanel.Align:=alTop;
    end;
    if not Assigned(FEditPanel) then begin
      FEditPanel:=TPanel.Create(Self);
      FEditPanel.Parent:=Self;
      FEditPanel.FreeNotification(Self);
      FEditPanel.Name:='EditPanel';
      FEditPanel.SetSubComponent(True);
      FEditPanel.Caption:='';
      FEditPanel.ParentBackground:=True;
      FEditPanel.ParentColor:=True;
      FEditPanel.BevelOuter:=bvNone;
      FEditPanel.Align:=alClient;
    end;
    if not Assigned(FButtonPanel) then begin
      FButtonPanel:=TPanel.Create(Self);
      FButtonPanel.Parent:=Self;
      FButtonPanel.FreeNotification(Self);
      FButtonPanel.Name:='ButtonPanel';
      FButtonPanel.SetSubComponent(True);
      FButtonPanel.Caption:='';
      FButtonPanel.ParentBackground:=True;
      FButtonPanel.ParentColor:=True;
      FButtonPanel.BevelOuter:=bvNone;
      FButtonPanel.Align:=alRight;
    end;
    if not Assigned(FSubEdit) then begin
      FSubEdit:=TEdit.Create(FEditPanel);
      FSubEdit.Parent:=FEditPanel;
      FSubEdit.FreeNotification(Self);
      FSubEdit.Name:='SubEdit';
      FSubEdit.SetSubComponent(True);
      FSubEdit.Text:='';
      FSubEdit.Align:=alTop;
    end;
    if not Assigned(FSubLabel) then begin
      FSubLabel:=TbdLabel.Create(FLabelPanel);
      FSubLabel.Parent:=FLabelPanel;
      FSubLabel.FreeNotification(Self);
      FSubLabel.FocusControl:=FSubEdit;
      FSubLabel.Name:='SubLabel';
      FSubLabel.SetSubComponent(True);
      FSubLabel.Caption:='Directory:';
      FSubLabel.OnResize:=SubLabelResize;
    end;
    if not Assigned(FSubButton) then begin
      FSubButton:=TbdButton.Create(FButtonPanel);
      FSubButton.Parent:=FButtonPanel;
      FSubButton.FreeNotification(Self);
      FSubButton.Name:='SubButton';
      FSubButton.SetSubComponent(True);
      FSubButton.AutoSize:=True;
      FSubButton.Caption:='&Browse';
      FSubButton.OnClick:=SubButtonClick;
      FSubButton.OnResize:=SubButtonResize;
    end;
    FSubLabelPosition:=bdpAbove;
    FSubButtonPosition:=bdpRight;
    SetLabelSpacing(3);
    SetButtonSpacing(8);
    if not Assigned(FSubDialog) then begin
      FSubDialog:=TSelectDirectoryDialog.Create(Self);
      FSubDialog.InitialDir:=SubDialogInitialDir;
      FSubDialog.FreeNotification(Self);
      FSubDialog.SetSubComponent(True);
      FSubDialog.Name:='SubDialog';
    end;
    FLabelPanel.ControlStyle:=FLabelPanel.ControlStyle-[csAcceptsControls];
    FEditPanel.ControlStyle:=FEditPanel.ControlStyle-[csAcceptsControls];
    FButtonPanel.ControlStyle:=FButtonPanel.ControlStyle-[csAcceptsControls];
    ControlStyle:=ControlStyle-[csAcceptsControls];
    Width:=305+FSubButtonSpacing+FSubButton.Width;
    Height:=FSubLabel.Height+FSubLabelSpacing+FSubButton.Height;
  end;

procedure TCustomBrowseDir.CMBidimodechanged(var Message: TMessage);
  begin
    inherited;
    FLabelPanel.BiDiMode:=BiDiMode;
    FEditPanel.BiDiMode:=BiDiMode;
    FButtonPanel.BiDiMode:=BiDiMode;
    FSubEdit.BiDiMode:=BiDiMode;
    FSubLabel.BiDiMode:=BiDiMode;
    FSubButton.BiDiMode:=BiDiMode;
  end;

procedure TCustomBrowseDir.CMEnabledchanged(var Message: TMessage);
  begin
    inherited;
    FLabelPanel.Enabled:=Enabled;
    FEditPanel.Enabled:=Enabled;
    FButtonPanel.Enabled:=Enabled;
    FSubEdit.Enabled:=Enabled;
    FSubLabel.Enabled:=Enabled;
    FSubButton.Enabled:=Enabled;
  end;

procedure TCustomBrowseDir.Notification(AComponent: TComponent; Operation: TOperation);
  begin
    inherited Notification(AComponent,Operation);
    if (Operation=opRemove) then begin
      if (AComponent=FLabelPanel) then FLabelPanel:=nil
      else if (AComponent=FEditPanel) then FEditPanel:=nil
      else if (AComponent=FButtonPanel) then FButtonPanel:=nil
      else if (AComponent=FSubEdit) then FSubEdit:=nil
      else if (AComponent=FSubLabel) then FSubLabel:=nil
      else if (AComponent=FSubButton) then FSubButton:=nil
      else if (AComponent=FSubDialog) then FSubDialog:=nil;
    end;
  end;

procedure TCustomBrowseDir.SetLabelPosition(const Value:TBrowseDirPosition1);
  begin
    if FSubLabel=nil then exit;
    FSubLabelPosition:=Value;
    FSubLabel.OnResize:=nil;
    case Value of
      bdpAbove : begin
        FLabelPanel.Align:=alTop;
        FLabelPanel.Height:=FSubLabel.Height+FSubLabelSpacing;
        FSubLabel.Top:=0;
        FSubLabel.Left:=0;
      end;
      bdpLeft : begin
        FLabelPanel.Align:=alLeft;
        FLabelPanel.Width:=FSubLabel.Width+FSubLabelSpacing;
        FSubLabel.Top:=(FSubEdit.Height-FSubLabel.Height) div 2;
        FSubLabel.Left:=0;
      end;
    end;
    FSubLabel.OnResize:=SubLabelResize;
  end;

procedure TCustomBrowseDir.SetLabelSpacing(const Value:Integer);
  begin
    FSubLabelSpacing:=Value;
    SetLabelPosition(FSubLabelPosition);
  end;

procedure TCustomBrowseDir.SetButtonPosition(const Value:TBrowseDirPosition2);
  begin
    if FSubButton=nil then exit;
    FSubButtonPosition:=Value;
    FSubButton.OnResize:=nil;
    case Value of
      bdpBelow : begin
        FSubButton.Parent:=FEditPanel;
        FSubButton.Top:=FSubEdit.Height+FSubButtonSpacing;
        FSubButton.Left:=0;
        FButtonPanel.Parent:=nil;
      end;
      bdpRight : begin
        FButtonPanel.Parent:=Self;
        FButtonPanel.Width:=FSubButton.Width+FSubButtonSpacing;
        FSubButton.Parent:=FButtonPanel;
        FSubButton.Top:=0;
        FSubButton.Left:=FSubButtonSpacing;
      end;
    end;
    FSubButton.OnResize:=SubButtonResize;
  end;

procedure TCustomBrowseDir.SetButtonSpacing(const Value:Integer);
  begin
    FSubButtonSpacing:=Value;
    SetButtonPosition(FSubButtonPosition);
  end;

procedure TCustomBrowseDir.SubLabelResize(Sender: TObject);
  begin
    if (FSubLabelPosition=bdpLeft) then SetLabelPosition(FSubLabelPosition);
  end;

procedure TCustomBrowseDir.SubButtonResize(Sender: TObject);
  begin
    if (FSubButtonPosition=bdpRight) then SetButtonPosition(FSubButtonPosition);
  end;

procedure TCustomBrowseDir.SubButtonClick(Sender: TObject);
  begin
    if Assigned(FOnBeforeShowDialog) then
      if (not FOnBeforeShowDialog(Self)) then exit;
    FSubDialog.Directory:=FSubEdit.Text;
    if FSubDialog.Execute then FSubEdit.Text:=FSubDialog.Directory;
  end;

procedure Register;
  begin
    RegisterComponents('Tools', [TBrowseDir]);
  end;

end.

