unit BrowseFile;

interface

uses
  Windows, SysUtils, Controls, Classes, Messages, StdCtrls, ExtCtrls,
  Dialogs, ExtDlgs, ASButton;

type
  TBrowseFilePosition1 = (bfpAbove, bfpLeft);
  TBrowseFilePosition2 = (bfpBelow, bfpRight);
  TBrowseFileDialogType = (bfdOpenDialog, bfdSaveDialog, bfdOpenPictureDialog, bfdSavePictureDialog);
  TOnBeforeShowDialog = function(Sender: TObject):boolean of object;
  TbfLabel = class(TLabel)
  public
    property OnResize;
  end;
  TbfButton = class(TASButton)
  public
    property OnResize;
  end;
  TCustomBrowseFile = class(TCustomPanel)
  private
    FSubLabel: TbfLabel;
    FSubLabelPosition: TBrowseFilePosition1;
    FSubLabelSpacing: Integer;
    FSubEdit: TEdit;
    FSubButton: TbfButton;
    FSubButtonPosition: TBrowseFilePosition2;
    FSubButtonSpacing: Integer;
    FLabelPanel: TPanel;
    FEditPanel: TPanel;
    FButtonPanel: TPanel;
    FSubOpenDialog: TOpenDialog;
    FSubSaveDialog: TSaveDialog;
    FSubOpenPictureDialog: TOpenPictureDialog;
    FSubSavePictureDialog: TSavePictureDialog;
    FUseSubDialog: TBrowseFileDialogType;
    FOnBeforeShowDialog: TOnBeforeShowDialog;
    procedure SetLabelPosition(const Value: TBrowseFilePosition1);
    procedure SetLabelSpacing(const Value: Integer);
    procedure SetButtonPosition(const Value: TBrowseFilePosition2);
    procedure SetButtonSpacing(const Value: Integer);
  protected
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    procedure CMEnabledchanged(var Message: TMessage); message CM_ENABLEDCHANGED;
    procedure CMBidimodechanged(var Message: TMessage); message CM_BIDIMODECHANGED;
    procedure SubLabelResize(Sender: TObject);
    procedure SubButtonResize(Sender: TObject);
    procedure SubButtonClick(Sender: TObject);
  public
    constructor Create(AOwner: TComponent); override;
    property SubLabel: TbfLabel read FSubLabel;
    property SubLabelPosition: TBrowseFilePosition1 read FSubLabelPosition write SetLabelPosition;
    property SubLabelSpacing: Integer read FSubLabelSpacing write SetLabelSpacing;
    property SubEdit: TEdit read FSubEdit;
    property SubButton: TbfButton read FSubButton;
    property SubButtonPosition: TBrowseFilePosition2 read FSubButtonPosition write SetButtonPosition;
    property SubButtonSpacing: Integer read FSubButtonSpacing write SetButtonSpacing;
    property SubOpenDialog: TOpenDialog read FSubOpenDialog;
    property SubSaveDialog: TSaveDialog read FSubSaveDialog;
    property SubOpenPictureDialog: TOpenPictureDialog read FSubOpenPictureDialog;
    property SubSavePictureDialog: TSavePictureDialog read FSubSavePictureDialog;
    property UseSubDialog: TBrowseFileDialogType read FUseSubDialog write FUseSubDialog;
    function GetSubDialog: TOpenDialog;
    property OnBeforeShowDialog: TOnBeforeShowDialog read FOnBeforeShowDialog write FOnBeforeShowDialog;
  end;
  TBrowseFile = class(TCustomBrowseFile)
  public
    property DockManager;
  published
    property Align;
    property Alignment;
    property Anchors;
    property AutoSize;
    property BevelInner;
    property BevelOuter;
    property BevelWidth;
    property BiDiMode;
    property BorderWidth;
    property BorderStyle;
    property Color;
    property Constraints;
    property Ctl3D;
    property UseDockManager default True;
    property DockSite;
    property DragCursor;
    property DragKind;
    property DragMode;
    property Enabled;
    property FullRepaint;
    property Font;
    property Locked;
    property ParentBiDiMode;
    property ParentBackground;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property SubLabel;
    property SubLabelPosition default bfpAbove;
    property SubLabelSpacing default 3;
    property SubEdit;
    property SubButton;
    property SubButtonPosition default bfpRight;
    property SubButtonSpacing default 3;
    property SubOpenDialog;
    property SubSaveDialog;
    property SubOpenPictureDialog;
    property SubSavePictureDialog;
    property UseSubDialog default bfdOpenDialog;
    property TabOrder;
    property TabStop;
    property Visible;
    property OnBeforeShowDialog;
    property OnCanResize;
    property OnClick;
    property OnConstrainedResize;
    property OnContextPopup;
    property OnDockDrop;
    property OnDockOver;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDock;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnGetSiteInfo;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnResize;
    property OnStartDock;
    property OnStartDrag;
    property OnUnDock;
  end;

procedure Register;

implementation

{$R *.res}

constructor TCustomBrowseFile.Create(AOwner: TComponent);
  const
    SubDialogFilter = 'All Files (*.*)|*.*';
    SubDialogInitialDir = '.';
  begin
    inherited Create(AOwner);
    Parent:=TWinControl(AOwner);
    Caption:='';
    BevelOuter:=bvNone;
    if not Assigned(FLabelPanel) then begin
      FLabelPanel:=TPanel.Create(Self);
      FLabelPanel.Parent:=Self;
      FLabelPanel.FreeNotification(Self);
      FLabelPanel.Name:='LabelPanel';
      FLabelPanel.SetSubComponent(True);
      FLabelPanel.Caption:='';
      FLabelPanel.ParentBackground:=True;
      FLabelPanel.ParentColor:=True;
      FLabelPanel.BevelOuter:=bvNone;
      FLabelPanel.Align:=alTop;
    end;
    if not Assigned(FEditPanel) then begin
      FEditPanel:=TPanel.Create(Self);
      FEditPanel.Parent:=Self;
      FEditPanel.FreeNotification(Self);
      FEditPanel.Name:='EditPanel';
      FEditPanel.SetSubComponent(True);
      FEditPanel.Caption:='';
      FEditPanel.ParentBackground:=True;
      FEditPanel.ParentColor:=True;
      FEditPanel.BevelOuter:=bvNone;
      FEditPanel.Align:=alClient;
    end;
    if not Assigned(FButtonPanel) then begin
      FButtonPanel:=TPanel.Create(Self);
      FButtonPanel.Parent:=Self;
      FButtonPanel.FreeNotification(Self);
      FButtonPanel.Name:='ButtonPanel';
      FButtonPanel.SetSubComponent(True);
      FButtonPanel.Caption:='';
      FButtonPanel.ParentBackground:=True;
      FButtonPanel.ParentColor:=True;
      FButtonPanel.BevelOuter:=bvNone;
      FButtonPanel.Align:=alRight;
    end;
    if not Assigned(FSubEdit) then begin
      FSubEdit:=TEdit.Create(FEditPanel);
      FSubEdit.Parent:=FEditPanel;
      FSubEdit.FreeNotification(Self);
      FSubEdit.Name:='SubEdit';
      FSubEdit.SetSubComponent(True);
      FSubEdit.Text:='';
      FSubEdit.Align:=alTop;
    end;
    if not Assigned(FSubLabel) then begin
      FSubLabel:=TbfLabel.Create(FLabelPanel);
      FSubLabel.Parent:=FLabelPanel;
      FSubLabel.FreeNotification(Self);
      FSubLabel.FocusControl:=FSubEdit;
      FSubLabel.Name:='SubLabel';
      FSubLabel.SetSubComponent(True);
      FSubLabel.Caption:='Filename:';
      FSubLabel.OnResize:=SubLabelResize;
    end;
    if not Assigned(FSubButton) then begin
      FSubButton:=TbfButton.Create(FButtonPanel);
      FSubButton.Parent:=FButtonPanel;
      FSubButton.FreeNotification(Self);
      FSubButton.Name:='SubButton';
      FSubButton.SetSubComponent(True);
      FSubButton.AutoSize:=True;
      FSubButton.Caption:='&Browse';
      FSubButton.OnClick:=SubButtonClick;
      FSubButton.OnResize:=SubButtonResize;
    end;
    FSubLabelPosition:=bfpAbove;
    FSubButtonPosition:=bfpRight;
    SetLabelSpacing(3);
    SetButtonSpacing(8);
    FUseSubDialog:=bfdOpenDialog;
    if not Assigned(FSubOpenDialog) then begin
      FSubOpenDialog:=TOpenDialog.Create(Self);
      FSubOpenDialog.Options:=FSubOpenDialog.Options-[ofHideReadOnly]+[ofFileMustExist];
      FSubOpenDialog.Filter:=SubDialogFilter;
      FSubOpenDialog.InitialDir:=SubDialogInitialDir;
      FSubOpenDialog.FreeNotification(Self);
      FSubOpenDialog.SetSubComponent(True);
      FSubOpenDialog.Name:='SubOpenDialog';
    end;
    if not Assigned(FSubSaveDialog) then begin
      FSubSaveDialog:=TSaveDialog.Create(Self);
      FSubSaveDialog.Options:=FSubSaveDialog.Options+[ofOverwritePrompt];
      FSubSaveDialog.Filter:=SubDialogFilter;
      FSubSaveDialog.InitialDir:=SubDialogInitialDir;
      FSubSaveDialog.FreeNotification(Self);
      FSubSaveDialog.SetSubComponent(True);
      FSubSaveDialog.Name:='SubSaveDialog';
    end;
    if not Assigned(FSubOpenPictureDialog) then begin
      FSubOpenPictureDialog:=TOpenPictureDialog.Create(Self);
      FSubOpenPictureDialog.Options:=FSubOpenPictureDialog.Options-[ofHideReadOnly]+[ofFileMustExist];
      FSubOpenPictureDialog.InitialDir:=SubDialogInitialDir;
      FSubOpenPictureDialog.FreeNotification(Self);
      FSubOpenPictureDialog.SetSubComponent(True);
      FSubOpenPictureDialog.Name:='SubOpenPictureDialog';
    end;
    if not Assigned(FSubSavePictureDialog) then begin
      FSubSavePictureDialog:=TSavePictureDialog.Create(Self);
      FSubSavePictureDialog.Options:=FSubSavePictureDialog.Options+[ofOverwritePrompt];
      FSubSavePictureDialog.InitialDir:=SubDialogInitialDir;
      FSubSavePictureDialog.FreeNotification(Self);
      FSubSavePictureDialog.SetSubComponent(True);
      FSubSavePictureDialog.Name:='SubSavePictureDialog';
    end;
    FLabelPanel.ControlStyle:=FLabelPanel.ControlStyle-[csAcceptsControls];
    FEditPanel.ControlStyle:=FEditPanel.ControlStyle-[csAcceptsControls];
    FButtonPanel.ControlStyle:=FButtonPanel.ControlStyle-[csAcceptsControls];
    ControlStyle:=ControlStyle-[csAcceptsControls];
    Width:=305+FSubButtonSpacing+FSubButton.Width;
    Height:=FSubLabel.Height+FSubLabelSpacing+FSubButton.Height;
  end;

procedure TCustomBrowseFile.CMBidimodechanged(var Message: TMessage);
  begin
    inherited;
    FLabelPanel.BiDiMode:=BiDiMode;
    FEditPanel.BiDiMode:=BiDiMode;
    FButtonPanel.BiDiMode:=BiDiMode;
    FSubEdit.BiDiMode:=BiDiMode;
    FSubLabel.BiDiMode:=BiDiMode;
    FSubButton.BiDiMode:=BiDiMode;
  end;

procedure TCustomBrowseFile.CMEnabledchanged(var Message: TMessage);
  begin
    inherited;
    FLabelPanel.Enabled:=Enabled;
    FEditPanel.Enabled:=Enabled;
    FButtonPanel.Enabled:=Enabled;
    FSubEdit.Enabled:=Enabled;
    FSubLabel.Enabled:=Enabled;
    FSubButton.Enabled:=Enabled;
  end;

procedure TCustomBrowseFile.Notification(AComponent: TComponent; Operation: TOperation);
  begin
    inherited Notification(AComponent,Operation);
    if (Operation=opRemove) then begin
      if (AComponent=FLabelPanel) then FLabelPanel:=nil
      else if (AComponent=FEditPanel) then FEditPanel:=nil
      else if (AComponent=FButtonPanel) then FButtonPanel:=nil
      else if (AComponent=FSubEdit) then FSubEdit:=nil
      else if (AComponent=FSubLabel) then FSubLabel:=nil
      else if (AComponent=FSubButton) then FSubButton:=nil
      else if (AComponent=FSubOpenDialog) then FSubOpenDialog:=nil
      else if (AComponent=FSubSaveDialog) then FSubSaveDialog:=nil
      else if (AComponent=FSubOpenPictureDialog) then FSubOpenPictureDialog:=nil
      else if (AComponent=FSubSavePictureDialog) then FSubSavePictureDialog:=nil;
    end;
  end;

procedure TCustomBrowseFile.SetLabelPosition(const Value:TBrowseFilePosition1);
  begin
    if FSubLabel=nil then exit;
    FSubLabelPosition:=Value;
    FSubLabel.OnResize:=nil;
    case Value of
      bfpAbove : begin
        FLabelPanel.Align:=alTop;
        FLabelPanel.Height:=FSubLabel.Height+FSubLabelSpacing;
        FSubLabel.Top:=0;
        FSubLabel.Left:=0;
      end;
      bfpLeft : begin
        FLabelPanel.Align:=alLeft;
        FLabelPanel.Width:=FSubLabel.Width+FSubLabelSpacing;
        FSubLabel.Top:=(FSubEdit.Height-FSubLabel.Height) div 2;
        FSubLabel.Left:=0;
      end;
    end;
    FSubLabel.OnResize:=SubLabelResize;
  end;

procedure TCustomBrowseFile.SetLabelSpacing(const Value:Integer);
  begin
    FSubLabelSpacing:=Value;
    SetLabelPosition(FSubLabelPosition);
  end;

procedure TCustomBrowseFile.SetButtonPosition(const Value:TBrowseFilePosition2);
  begin
    if FSubButton=nil then exit;
    FSubButtonPosition:=Value;
    FSubButton.OnResize:=nil;
    case Value of
      bfpBelow : begin
        FSubButton.Parent:=FEditPanel;
        FSubButton.Top:=FSubEdit.Height+FSubButtonSpacing;
        FSubButton.Left:=0;
        FButtonPanel.Parent:=nil;
      end;
      bfpRight : begin
        FButtonPanel.Parent:=Self;
        FButtonPanel.Width:=FSubButton.Width+FSubButtonSpacing;
        FSubButton.Parent:=FButtonPanel;
        FSubButton.Top:=0;
        FSubButton.Left:=FSubButtonSpacing;
      end;
    end;
    FSubButton.OnResize:=SubButtonResize;
  end;

procedure TCustomBrowseFile.SetButtonSpacing(const Value:Integer);
  begin
    FSubButtonSpacing:=Value;
    SetButtonPosition(FSubButtonPosition);
  end;

function TCustomBrowseFile.GetSubDialog:TOpenDialog;
  begin
    case FUseSubDialog of
      bfdOpenDialog : Result:=FSubOpenDialog;
      bfdSaveDialog : Result:=FSubSaveDialog;
      bfdOpenPictureDialog : Result:=FSubOpenPictureDialog;
      bfdSavePictureDialog : Result:=FSubSavePictureDialog;
    end;
  end;

procedure TCustomBrowseFile.SubLabelResize(Sender: TObject);
  begin
    if (FSubLabelPosition=bfpLeft) then SetLabelPosition(FSubLabelPosition);
  end;

procedure TCustomBrowseFile.SubButtonResize(Sender: TObject);
  begin
    if (FSubButtonPosition=bfpRight) then SetButtonPosition(FSubButtonPosition);
  end;

procedure TCustomBrowseFile.SubButtonClick(Sender: TObject);
  var
    FSubDialog : TOpenDialog;
  begin
    if Assigned(FOnBeforeShowDialog) then
      if (not FOnBeforeShowDialog(Self)) then exit;
    FSubDialog:=GetSubDialog;
    FSubDialog.FileName:=FSubEdit.Text;
    if FSubDialog.Execute then FSubEdit.Text:=FSubDialog.FileName;
  end;

procedure Register;
  begin
    RegisterComponents('Tools', [TBrowseFile]);
  end;

end.

