{$WARN SYMBOL_PLATFORM OFF}
{$HINTS OFF}
unit Littles;

interface

uses
  Forms, SysUtils, Classes, Windows, Graphics;

const
  faFile = faAnyFile-faVolumeID-faDirectory-faHidden-faSysFile;
  faFileAny = faAnyFile-faVolumeID-faDirectory;

type
  Binary = array of byte;
  FileSearches = array of TSearchRec;
  TPipeStream = class(THandleStream)
  public
    function NBRead(var Buffer; Count: Longint): Longint; virtual;  // Non-Blocking-Read
    procedure EmptyPipe(var Buf:array of byte);
    procedure PipeToPipe(DestinationPipe:TPipeStream; var Buf:array of byte);
  end;
  TStringListOnPipeStream = class(TStringList)
  protected
    PipeStream:TPipeStream;
    procedure MySetTextStr(const Value: string);
    procedure RefreshFromStream;
    function Get(Index: Integer): string; override;
    function GetCount: Integer; override;
  public
    constructor Create(PipeStream:TPipeStream);
  end;
  TCapturedProcessStruct = class
  protected
    StdInReadPipe,StdInWritePipe : THandle;
    StdOutReadPipe,StdOutWritePipe : THandle;
    StdErrReadPipe,StdErrWritePipe : THandle;
  public
    ProcessInformation : TProcessInformation;
    StdIn : TPipeStream;
    StdOut : TPipeStream;
    StdErr : TPipeStream;
    StdOutSL : TStringListOnPipeStream;
    StdErrSL : TStringListOnPipeStream;
    constructor Create(PipeSize:DWORD);
    destructor Destroy; override;
  end;

function UC(const s:string): string;
function LC(const s:string): string;
function RndInt(l,h:longint):longint;
function OSisNT:boolean;
function IsNumber(const s:string):boolean;
function IsRealNumber(const s:string):boolean;
function IsHexNumber(const s:string):boolean;
function StrToFloat2(s:string):extended;
function FloatToStr2(n:extended;Width,Decimals:byte):string;
function IntToStr2(Value:longint;Width:byte):string;
function HexToInt(s:string):longint;
function BinaryToHex(const a:Binary):string;
function HexToBinary(s:string):Binary;
function ByteToBinary(b:byte):string;
function BinaryToByte(s:string):byte;
function BinaryToWord(s:string):word;
function BinaryToDWord(s:string):dword;
procedure SpaceLess(var s:string);
procedure SpaceFLess(var s:string);
procedure ConvToEnt(c:char;var s:string;const x:string);
function OName(const s:string):string;
function OPath(const s:string):string;
function ESPathName(const FileName,NonValue:string):string;
function WithoutExt(const s:string):string;
function MakeAFName(const ExeDir,s:string):string; // Make Absolute FileName (to the EXE)
function MakeRFName(const ExeDir,s:string):string; // Make Relative FileName (to the EXE)
procedure FindFiles(const PathMask:string; Attr:integer; var a:FileSearches);
procedure DeleteFiles(const PathMask:string);
function GetLogicalCpuCount:integer;
function ExecuteFile(const FileName,Parameters:string;ShowCmd:Integer):longint;
function ExecuteProcess(const FileName,Parameters,StartDir:string; ShowCmd:Integer; Handles:byte):TProcessInformation;
function ExecuteCaptured(const FileName,Parameters,StartDir:string; PipeSize:DWORD):TCapturedProcessStruct;
procedure WaitProcess(const Process:TProcessInformation; timeout,sleeptime:DWORD);
function IsProcessExists(PID:DWORD):boolean;
function IsGuid(const s: string): boolean;
function GuidToStr(Guid:TGUID):string;
function StrToGuid(const s:string):TGUID;
function CompareGuids(const G1,G2:TGUID):boolean;
function CompVideo(const G:TGUID):boolean;
function CompAudio(const G:TGUID):boolean;
procedure FormToCenter(Form:TForm);
function GetIPs:TStrings;
function GetIP:string;
function GetHost:string;
function IPtoName(IPAddr: string): string;
function CompareBitmaps(Bmp1,Bmp2:TBitmap):Extended; // Result: difference in percent, works with 24b, 32b
function BitmapsDiffSmallerThan(Bmp1,Bmp2:TBitmap; const Percent:Extended):boolean; // works with 24b, 32b

implementation

uses
  ShellApi, Winsock;

{ TPipeStream }

function TPipeStream.NBRead(var Buffer; Count: Integer): Longint;
  var
    data_available : boolean;
    Buf : byte;
    BytesRead,TBA,BLTM : DWORD;
  begin
    {$IFDEF MSWINDOWS}
      data_available:=(PeekNamedPipe(THandle(FHandle),@Buf,1,@BytesRead,@TBA,@BLTM) and (BytesRead>0));
    {$ENDIF}
    {$IFDEF LINUX}
      data_available:=true; // Not implemented
    {$ENDIF}
    if data_available then Result:=inherited Read(Buffer,Count) else Result:=0;
  end;

procedure TPipeStream.EmptyPipe(var Buf:array of byte);
  var
    S : integer;
  begin
    repeat
      S:=Size-Position;
      if (S>0) then begin
        if (S>sizeof(Buf)) then S:=sizeof(Buf);
        NBRead(Buf[0],S);
      end;
    until S<sizeof(Buf);
  end;

procedure TPipeStream.PipeToPipe(DestinationPipe:TPipeStream; var Buf:array of byte);
  var
    S,c : integer;
  begin
    repeat
      S:=Size-Position;
      if (S>0) then begin
        if (S>sizeof(Buf)) then S:=sizeof(Buf);
        c:=NBRead(Buf[0],S);
        DestinationPipe.Write(Buf[0],c);
      end;
    until S<sizeof(Buf);
  end;

{ TStringListOnPipeStream }

constructor TStringListOnPipeStream.Create(PipeStream: TPipeStream);
  begin
    self.PipeStream:=PipeStream;
  end;

function TStringListOnPipeStream.Get(Index: Integer): string;
  begin
    RefreshFromStream;
    Result:=inherited Get(Index);
  end;

function TStringListOnPipeStream.GetCount: Integer;
  begin
    RefreshFromStream;
    Result:=inherited GetCount;
  end;

procedure TStringListOnPipeStream.MySetTextStr(const Value: string);
  var
    P,Start : PChar;
    S : string;
  begin
    BeginUpdate;
    try
      P:=Pointer(Value);
      if (P<>nil) then
        while (P^<>#0) do begin
          Start:=P;
          while (not (P^ in [#0,#10,#13])) do Inc(P);
          SetString(S,Start,P-Start);
          Strings[(inherited GetCount)-1]:=(inherited Get((inherited GetCount)-1))+S;
          if (P^=#13) then begin
            Inc(P);
            if (P^<>#10) then Add('');
          end;
          if (P^=#10) then begin Inc(P); Add(''); end;
        end;
    finally
      EndUpdate;
    end;
  end;

procedure TStringListOnPipeStream.RefreshFromStream;
  var
    Size : Integer;
    S : string;
  begin
    BeginUpdate;
    try
      Size:=PipeStream.Size-PipeStream.Position;
      if (Size>0) then begin
        SetString(S,nil,Size);
        if (PipeStream.NBRead(Pointer(S)^,Size)>0) then MySetTextStr(S);
      end;
    finally
      EndUpdate;
    end;
  end;

{ TCapturedProcessStruct }

constructor TCapturedProcessStruct.Create(PipeSize:DWORD);
  var
    Security : TSecurityAttributes;
  begin
    inherited Create;
    FillChar(ProcessInformation,SizeOf(ProcessInformation),#0);
    with Security do begin
      nlength:=SizeOf(TSecurityAttributes);
      binherithandle:=true;
      lpsecuritydescriptor:=nil;
    end;
    if (not CreatePipe(StdInReadPipe,StdInWritePipe,@Security,PipeSize)) then begin
      StdInReadPipe:=0; StdInWritePipe:=0;
    end;
    if (not CreatePipe(StdOutReadPipe,StdOutWritePipe,@Security,PipeSize)) then begin
      StdOutReadPipe:=0; StdOutWritePipe:=0;
    end;
    if (not CreatePipe(StdErrReadPipe,StdErrWritePipe,@Security,PipeSize)) then begin
      StdErrReadPipe:=0; StdErrWritePipe:=0;
    end;
    StdIn:=TPipeStream.Create(StdInWritePipe);
    StdOut:=TPipeStream.Create(StdOutReadPipe);
    StdErr:=TPipeStream.Create(StdErrReadPipe);
    StdOutSL:=TStringListOnPipeStream.Create(StdOut); StdOutSL.Add('');
    StdErrSL:=TStringListOnPipeStream.Create(StdErr); StdErrSL.Add('');
  end;

destructor TCapturedProcessStruct.Destroy;
  begin
    if (0<>ProcessInformation.hThread) then CloseHandle(ProcessInformation.hThread);
    if (0<>ProcessInformation.hProcess) then CloseHandle(ProcessInformation.hProcess);
    StdOutSL.Destroy; StdErrSL.Destroy;
    StdIn.Destroy; StdOut.Destroy; StdErr.Destroy;
    if (0<>StdInReadPipe) then CloseHandle(StdInReadPipe);
    if (0<>StdInWritePipe) then CloseHandle(StdInWritePipe);
    if (0<>StdOutReadPipe) then CloseHandle(StdOutReadPipe);
    if (0<>StdOutWritePipe) then CloseHandle(StdOutWritePipe);
    if (0<>StdErrReadPipe) then CloseHandle(StdErrReadPipe);
    if (0<>StdErrWritePipe) then CloseHandle(StdErrWritePipe);
    inherited;
  end;

{ Static Functions }

var
  isnotNTGot,isNT : boolean;

function UC(const s:string): string;
  begin  Result:=UpperCase(s);  end;
function LC(const s:string): string;
  begin  Result:=LowerCase(s);  end;
function RndInt(l,h:longint):longint;
  begin  Result:=l+Random(h-l+1);  end;

function OSisNT:boolean;
  var
    verinfo : OSVERSIONINFO;
  begin
    if isnotNTGot then begin
      verinfo.dwOSVersionInfoSize:=sizeof(verinfo);
      GetVersionEx(verinfo);
      isNT:=(verinfo.dwPlatformId=VER_PLATFORM_WIN32_NT);
      isnotNTGot:=FALSE;
    end;
    Result:=isNT;
  end;

function IsNumber(const s:string):boolean;
  var
    v,code : integer;
  begin
    val(s,v,code); result:=(code=0);
  end;

function IsRealNumber(const s:string):boolean;
  var
    v : real;
    code : integer;
  begin
    val(s,v,code); result:=(code=0);
  end;

function IsHexNumber(const s:string):boolean;
  var
    i : longint;
  begin
    Result:=TRUE;
    for i:=1 to length(s) do
      Result:=Result and (upcase(s[i]) in ['A'..'F','0'..'9']);
  end;

function StrToFloat2(s:string):extended;
  var
    code : integer;
  begin
    val(s,Result,code);
  end;

function FloatToStr2(n:extended;Width,Decimals:byte):string;
  var
    s : string;
  begin
    Str(n:Width:Decimals,s);
    Result:=s;
  end;

function IntToStr2(Value:longint;Width:byte):string;
  begin
    Result:=IntToStr(Value);
    while length(Result)<Width do Result:='0'+Result;
  end;

function Getnum(a:char):byte;
  begin
    if a in ['0'..'9'] then result:=byte(a)-$30
    else if a in ['A'..'F'] then result:=byte(a)-55
    else result:=0;
  end;

function HexToInt(s:string):longint;
  begin
    Result:=0;
    while length(s)>0 do begin
      Result:=(Result shl 4) or Getnum(upcase(s[1]));
      delete(s,1,1);
    end;
  end;

function BinaryToHex(const a:Binary):string;
  const
    h : array[0..15] of char = (
      '0','1','2','3','4','5','6','7','8','9','A','B','C','D','E','F');
  var
    i : longint;
  begin
    Result:='';
    for i:=low(a) to high(a) do Result:=Result+h[a[i] shr 4]+h[a[i] and 15];
  end;

function HexToBinary(s:string):Binary;
  begin
    setlength(Result,0);
    while length(s)>=2 do begin
      setlength(Result,length(Result)+1);
      Result[high(Result)]:=(Getnum(upcase(s[1])) shl 4) or Getnum(upcase(s[2]));
      delete(s,1,2);
    end;
  end;

function ByteToBinary(b:byte):string;
  begin
    Result:='';
    if (b and 128)=0 then Result:=Result+'0' else Result:=Result+'1';
    if (b and 64)=0 then Result:=Result+'0' else Result:=Result+'1';
    if (b and 32)=0 then Result:=Result+'0' else Result:=Result+'1';
    if (b and 16)=0 then Result:=Result+'0' else Result:=Result+'1';
    if (b and 8)=0 then Result:=Result+'0' else Result:=Result+'1';
    if (b and 4)=0 then Result:=Result+'0' else Result:=Result+'1';
    if (b and 2)=0 then Result:=Result+'0' else Result:=Result+'1';
    if (b and 1)=0 then Result:=Result+'0' else Result:=Result+'1';
  end;

function BinaryToByte(s:string):byte;
  var
    bit : byte;
  begin
    Result:=0; bit:=1;
    while (length(s)>0) and (not (s[length(s)] in ['0','1']) ) do delete(s,length(s),1);
    while (length(s)>0) and (bit<>0) do begin
      case s[length(s)] of
        '0' : ;
        '1' : Result:=Result or bit;
      end;
      delete(s,length(s),1);
      if bit<$80 then bit:=bit*2 else bit:=0;
      while (length(s)>0) and (not (s[length(s)] in ['0','1']) ) do delete(s,length(s),1);
    end;
  end;

function BinaryToWord(s:string):word;
  var
    bit : word;
  begin
    Result:=0; bit:=1;
    while (length(s)>0) and (not (s[length(s)] in ['0','1']) ) do delete(s,length(s),1);
    while (length(s)>0) and (bit<>0) do begin
      case s[length(s)] of
        '0' : ;
        '1' : Result:=Result or bit;
      end;
      delete(s,length(s),1);
      if bit<$8000 then bit:=bit*2 else bit:=0;
      while (length(s)>0) and (not (s[length(s)] in ['0','1']) ) do delete(s,length(s),1);
    end;
  end;

function BinaryToDWord(s:string):dword;
  var
    bit : dword;
  begin
    Result:=0; bit:=1;
    while (length(s)>0) and (not (s[length(s)] in ['0','1']) ) do delete(s,length(s),1);
    while (length(s)>0) and (bit<>0) do begin
      case s[length(s)] of
        '0' : ;
        '1' : Result:=Result or bit;
      end;
      delete(s,length(s),1);
      if bit<$80000000 then bit:=bit*2 else bit:=0;
      while (length(s)>0) and (not (s[length(s)] in ['0','1']) ) do delete(s,length(s),1);
    end;
  end;

procedure SpaceLess(var s:string);
  begin
    while (length(s)>0) and (s[length(s)] in [' ',#09]) do delete(s,length(s),1);
    SpaceFLess(s);
  end;

procedure SpaceFLess(var s:string);
  begin
    while (length(s)>0) and (s[1] in [' ',#09]) do delete(s,1,1);
  end;

procedure ConvToEnt(c:char;var s:string;const x:string);
  var
    i : longint;
  begin
    i:=pos(c,s);
    while i<>0 do begin
      delete(s,i,1);
      insert(x,s,i);
      i:=pos(c,s);
    end;
  end;

function OName(const s:string):string;
  var
    i : longint;
  begin
    i:=length(s);
    while ((i>0) and (s[i]<>'\')) do dec(i);
    if (i>0) then Result:=copy(s,i+1,maxint) else Result:=s;
  end;

function OPath(const s:string):string;
  var
    i : longint;
  begin
    i:=length(s);
    while ((i>0) and (s[i]<>'\')) do dec(i);
    Result:=copy(s,1,i);
  end;

function ESPathName(const FileName,NonValue:string):string;
  begin
    Result:=ExtractShortPathName(FileName);
    if Result='' then Result:=NonValue;
  end;

function WithoutExt(const s:string):string;
  var
    i : longint;
  begin
    i:=length(s);
    while (i>0) and (s[i]<>'.') do dec(i);
    if (i>0) then Result:=copy(s,1,i-1) else Result:=s;
  end;

function MakeAFName(const ExeDir,s:string):string;
  var
    i,j : longint;
  begin if length(s)=0 then Result:='' else begin
    if ((length(s)<2) or (s[2]<>':')) and (s[1]<>'\') then Result:=exedir+s
    else if (length(s)>1) and (s[2]=':') and (s[1]<>'\') then Result:=s
    else if (s[1]='\') then Result:=copy(exedir,1,2)+s
    else Result:='';
    i:=pos('\..\',Result);
    while i<>0 do begin
      j:=i-1;
      while (j>0) and (Result[j]<>'\') do dec(j);
      if j>0 then Delete(Result,j,i-j+3);
      i:=pos('\..\',Result);
    end;
  end; end;

function MakeRFName(const ExeDir,s:string):string;
  var
    i,j : longint;
    ss : string;
  function BackSlashesCount(sss:string):longint;
    begin
      Result:=0;
      while pos('\',sss)<>0 do begin
        delete(sss,1,pos('\',sss));
        inc(Result);
      end;
    end;
  begin if length(s)=0 then Result:='' else begin
    if UpCase(s[1])<>UpCase(exedir[1]) then Result:=s else begin
      ss:=exedir; Result:=s;
      Delete(ss,1,3); Delete(Result,1,3);
      i:=pos('\',ss); j:=pos('\',Result);
      if (i=0) or (i<>j) or (UC(copy(ss,1,i))<>UC(copy(Result,1,j)))
        then Result:='\'+Result else begin
          repeat
            Delete(ss,1,i); Delete(Result,1,j);
            i:=pos('\',ss); j:=pos('\',Result);
          until (i=0) or (i<>j) or (UC(copy(ss,1,i))<>UC(copy(Result,1,j)));
          for i:=1 to BackSlashesCount(ss) do Result:='..\'+Result;
      end;
    end;
  end; end;

procedure FindFiles(const PathMask:string; Attr:integer; var a:FileSearches);
  var
    F : TSearchRec;
  begin
    setlength(a,0);
    if SysUtils.FindFirst(PathMask,Attr,F)=0 then repeat
      setlength(a,length(a)+1);
      a[high(a)]:=F;
    until SysUtils.FindNext(F)<>0;
    SysUtils.FindClose(F);
  end;

procedure DeleteFiles(const PathMask:string);
  var
    files : FileSearches;
    i : integer;
    Path : string;
  begin
    FindFiles(PathMask,faReadOnly+faHidden+faSysFile+faArchive,files);
    Path:=OPath(PathMask);
    for i:=low(files) to high(files) do DeleteFile(PChar(Path+files[i].Name));
  end;

function GetLogicalCpuCount:integer;
  var
    SystemInfo : _SYSTEM_INFO;
  begin
    GetSystemInfo(SystemInfo);
    Result:=SystemInfo.dwNumberOfProcessors;
  end;

function ExecuteFile(const FileName,Parameters:string;ShowCmd:Integer):longint;
  var zFileName,zParam  : array[0..255] of Char;
  begin
    Result:=ShellExecute(GetDesktopWindow,nil,StrPCopy(zFileName,FileName),
             StrPCopy(zParam,Parameters),nil,ShowCmd);
  end;

function ExecuteProcess(const FileName,Parameters,StartDir:string; ShowCmd:Integer; Handles:byte):TProcessInformation;
  var
    start : TStartupInfo;
  begin
    try
      FillChar(start,SizeOf(start),#0);
      start.cb:=SizeOf(start);
      start.dwFlags:=StartF_UseShowWindow;
      start.wShowWindow:=ShowCmd;
      if CreateProcess(nil,PChar(FileName+' '+Parameters),nil,nil,True,
        0,nil,PChar(StartDir),start,Result) then begin
          case Handles of
            0 : begin // User doesn't need any of the Handles
              CloseHandle(Result.hThread); Result.hThread:=0;
              CloseHandle(Result.hProcess); Result.hProcess:=0;
            end;
            1 : begin // User needs only the Process Handle
              CloseHandle(Result.hThread); Result.hThread:=0;
            end;
            2 : begin // User needs only the Thread Handle
              CloseHandle(Result.hProcess); Result.hProcess:=0;
            end;
            else ; // User needs all Handles
          end;
        end else FillChar(Result,SizeOf(Result),#0);
    except FillChar(Result,SizeOf(Result),#0); end;
  end;

function ExecuteCaptured(const FileName,Parameters,StartDir:string; PipeSize:DWORD):TCapturedProcessStruct;
  var
    start : TStartupInfo;
  begin
    Result:=TCapturedProcessStruct.Create(PipeSize);
    FillChar(start,SizeOf(start),#0);
    start.cb:=SizeOf(start);
    start.hStdInput:=Result.StdInReadPipe;
    start.hStdOutput:=Result.StdOutWritePipe;
    start.hStdError:=Result.StdErrWritePipe;
    start.dwFlags:=StartF_UseStdHandles or StartF_UseShowWindow;
    start.wShowWindow:=SW_HIDE;
    if (not CreateProcess(nil,PChar(FileName+' '+Parameters),nil,nil,True,
      0,nil,PChar(StartDir),start,Result.ProcessInformation)) then begin
        Result.ProcessInformation.hThread:=0;
        Result.ProcessInformation.hProcess:=0;
        Result.Destroy;
        Result:=nil;
      end;
  end;

procedure WaitProcess(const Process:TProcessInformation; timeout,sleeptime:DWORD);
  var
    start : DWORD;
  begin
    start:=GetTickCount;
    repeat  Application.ProcessMessages;
    until (WaitForSingleObject(Process.hProcess,sleeptime)<>WAIT_TIMEOUT) or ((GetTickCount-start)>=timeout);
    CloseHandle(Process.hProcess);
  end;

function IsProcessExists(PID:DWORD):boolean;
  var
//    cb: DWORD;
//    hMod: HMODULE;
    hProcess: THandle;
//    ModuleName: array [0..300] of Char;
  begin
    Result:=FALSE;
    hProcess:=OpenProcess(PROCESS_QUERY_INFORMATION or PROCESS_VM_READ,FALSE,PID);
    if hProcess<>0 then begin
      Result:=TRUE;
      CloseHandle(hProcess);
    end;
  end;

function IsGuid(const s: string): boolean;
  begin
    Result:=(length(s)=38) and (s[1]='{') and (s[38]='}')
      and (s[10]='-') and (s[15]='-') and (s[20]='-') and (s[25]='-')
      and IsHexNumber(copy(s,2,8)) and IsHexNumber(copy(s,11,4)) and IsHexNumber(copy(s,16,4))
      and IsHexNumber(copy(s,21,4)) and IsHexNumber(copy(s,26,12));
  end;

function GuidToStr(Guid:TGUID):string;
  begin
    Result:='{'+inttohex(Guid.D1,8)+'-'+inttohex(Guid.D2,4)+'-'
      +inttohex(Guid.D3,4)+'-'+inttohex(Guid.D4[0],2)+inttohex(Guid.D4[1],2)+'-'
      +inttohex(Guid.D4[2],2)+inttohex(Guid.D4[3],2)+inttohex(Guid.D4[4],2)
      +inttohex(Guid.D4[5],2)+inttohex(Guid.D4[6],2)+inttohex(Guid.D4[7],2)+'}';
  end;

function StrToGuid(const s:string):TGUID;
  begin
    Result.D1:=HexToInt(copy(s,2,8));
    Result.D2:=HexToInt(copy(s,11,4));
    Result.D3:=HexToInt(copy(s,16,4));
    Result.D4[0]:=HexToInt(copy(s,21,2));
    Result.D4[1]:=HexToInt(copy(s,23,2));
    Result.D4[2]:=HexToInt(copy(s,26,2));
    Result.D4[3]:=HexToInt(copy(s,28,2));
    Result.D4[4]:=HexToInt(copy(s,30,2));
    Result.D4[5]:=HexToInt(copy(s,32,2));
    Result.D4[6]:=HexToInt(copy(s,34,2));
    Result.D4[7]:=HexToInt(copy(s,36,2));
  end;

function CompareGuids(const G1,G2:TGUID):boolean;
  begin
    Result:=(G1.D1=G2.D1) and (G1.D2=G2.D2) and (G1.D3=G2.D3)
      and (G1.D4[0]=G2.D4[0]) and (G1.D4[1]=G2.D4[1])
      and (G1.D4[2]=G2.D4[2]) and (G1.D4[3]=G2.D4[3])
      and (G1.D4[4]=G2.D4[4]) and (G1.D4[5]=G2.D4[5])
      and (G1.D4[6]=G2.D4[6]) and (G1.D4[7]=G2.D4[7]);
  end;

function CompVideo(const G:TGUID):boolean;
  var
    AG : array[0..15] of char absolute G;
  begin
    Result:=(upcase(AG[0]) in ['A'..'Z','0'..'9'])
      and (upcase(AG[1]) in ['A'..'Z','0'..'9'])
      and (upcase(AG[2]) in ['A'..'Z','0'..'9'])
      and (upcase(AG[3]) in ['A'..'Z','0'..'9'])
      and (G.D2=$0000) and (G.D3=$0010) and (G.D4[0]=$80) and (G.D4[1]=$00)
      and (G.D4[2]=$00) and (G.D4[3]=$AA)
      and (G.D4[4]=$00) and (G.D4[5]=$38)
      and (G.D4[6]=$9B) and (G.D4[7]=$71);
  end;

function CompAudio(const G:TGUID):boolean;
  begin
    Result:=(G.D2=$0000) and (G.D3=$0010) and (G.D4[0]=$80) and (G.D4[1]=$00)
      and (G.D4[2]=$00) and (G.D4[3]=$AA)
      and (G.D4[4]=$00) and (G.D4[5]=$38)
      and (G.D4[6]=$9B) and (G.D4[7]=$71);
  end;

procedure FormToCenter(Form:TForm);
  begin with Form do begin
    if Height>Screen.WorkAreaHeight then Height:=Screen.WorkAreaHeight;
    if Width>Screen.WorkAreaWidth then Width:=Screen.WorkAreaWidth;
    Top:=(Screen.WorkAreaHeight div 2)-(Height div 2);
    if Top<0 then Top:=0;
    Left:=(Screen.WorkAreaWidth div 2)-(Width div 2);
    if Left<0 then Left:=0;
  end; end;

function GetIPs:TStrings;
  type
    TaPInAddr = array[0..10] of PInAddr;
    PaPInAddr = ^TaPInAddr;
  var
    phe       : PHostEnt;
    pptr      : PaPInAddr;
    Buffer    : array[0..63] of Char;
    i         : Integer;
    GInitData : TWSAData;
  begin
    WSAStartup($101,GInitData);
    Result:=TStringList.Create;
    Result.Clear;
    GetHostName(Buffer,SizeOf(Buffer));
    phe:=GetHostByName(buffer);
    if phe=nil then Exit;
    pPtr:=PaPInAddr(phe^.h_addr_list);
    i:=0;
    while pPtr^[i]<>nil do begin
      Result.Add(inet_ntoa(pptr^[i]^));
      Inc(i);
    end;
    WSACleanup;
  end;

function GetIP:string;
  type
    TaPInAddr = array[0..10] of PInAddr;
    PaPInAddr = ^TaPInAddr;
  var
    phe       : PHostEnt;
    pptr      : PaPInAddr;
    Buffer    : array[0..63] of Char;
    i         : Integer;
    s         : string;
    GInitData : TWSAData;
  begin
    Result:='';
    WSAStartup($101,GInitData);
    GetHostName(Buffer,SizeOf(Buffer));
    phe:=GetHostByName(buffer);
    if phe=nil then Exit;
    pPtr:=PaPInAddr(phe^.h_addr_list);
    i:=0;
    while (Result='') and (pPtr^[i]<>nil) do begin
      s:=inet_ntoa(pPtr^[i]^);
      if (s<>'') and (copy(s,1,5)<>'10.0.') and (copy(s,1,8)<>'192.168.')
        then Result:=s;
      Inc(i);
    end;
    WSACleanup;
  end;

function GetHost:string;
  var
    CPS : TCapturedProcessStruct;
    i : longint;
    IP : string;
  begin
    IP:=GetIP;
    CPS:=ExecuteCaptured('nslookup',IP,'C:\',0);
    WaitForSingleObject(CPS.ProcessInformation.hProcess,Infinite);
    i:=CPS.StdOutSL.Count-1;
    while (i>=0) and (uc(copy(CPS.StdOutSL[i],1,5))<>uc('Name:')) do dec(i);
    if i<0 then Result:=IP else Result:=copy(CPS.StdOutSL[i],10,length(CPS.StdOutSL[i])-9);
    CPS.Destroy;
  end;

function IPtoName(IPAddr: string): string;
  var
    SockAddrIn: TSockAddrIn;
    HostEnt: PHostEnt;
    WSAData: TWSAData;
  begin
    WSAStartup($101, WSAData);
    SockAddrIn.sin_addr.s_addr:=inet_addr(PChar(IPAddr));
    HostEnt:=gethostbyaddr(@SockAddrIn.sin_addr.S_addr,4,AF_INET);
    if HostEnt<>nil
      then Result:=StrPas(Hostent^.h_name)
      else Result:='';
    WSACleanup;
  end;

function CompareBitmaps(Bmp1,Bmp2:TBitmap):Extended;
  var
    i,j,m1,m2 : DWORD;
    L1,L2 : DWORD;
    pix1,pix2 : TColor;
  begin
    if (Bmp1.Width<>Bmp2.Width) or (Bmp1.Height<>Bmp2.Height) then
      raise Exception.Create('CompareBitmaps(): Bitmaps must have identical dimensions!');
    if (Bmp1.PixelFormat=pf32bit) then m1:=4 else m1:=3;
    if (Bmp2.PixelFormat=pf32bit) then m2:=4 else m2:=3;
    Result:=0;
    for i:=0 to Bmp1.Height-1 do begin
      L1:=DWORD(Bmp1.ScanLine[i]);
      L2:=DWORD(Bmp2.ScanLine[i]);
      for j:=0 to Bmp1.Width-1 do begin
        try
          pix1:=PColor(L1+j*m1)^;
          pix2:=PColor(L2+j*m2)^;
          Result:=Result+abs(GetRValue(pix1)-GetRValue(pix2))
                        +abs(GetGValue(pix1)-GetGValue(pix2))
                        +abs(GetBValue(pix1)-GetBValue(pix2));
        except end;
      end;
    end;
    Result:=(Result/255*100)/(Bmp1.Width*Bmp1.Height*3);
  end;

function BitmapsDiffSmallerThan(Bmp1,Bmp2:TBitmap; const Percent:Extended):boolean;
  var
    totaldiff, maxdiff : Extended;
    i,j,m1,m2 : DWORD;
    L1,L2 : DWORD;
    pix1,pix2 : TColor;
  begin
    if (Bmp1.Width<>Bmp2.Width) or (Bmp1.Height<>Bmp2.Height) then
      raise Exception.Create('BitmapsDiffSmallerThan(): Bitmaps must have identical dimensions!');
    maxdiff:=((Percent*(Bmp1.Width*Bmp1.Height*3))/100)*255;
    if (Bmp1.PixelFormat=pf32bit) then m1:=4 else m1:=3;
    if (Bmp2.PixelFormat=pf32bit) then m2:=4 else m2:=3;
    totaldiff:=0;
    for i:=0 to Bmp1.Height-1 do begin
      L1:=DWORD(Bmp1.ScanLine[i]);
      L2:=DWORD(Bmp2.ScanLine[i]);
      for j:=0 to Bmp1.Width-1 do begin
        try
          pix1:=PColor(L1+j*m1)^;
          pix2:=PColor(L2+j*m2)^;
          totaldiff:=totaldiff+abs(GetRValue(pix1)-GetRValue(pix2))
                              +abs(GetGValue(pix1)-GetGValue(pix2))
                              +abs(GetBValue(pix1)-GetBValue(pix2));
        except end;
        if (totaldiff>=maxdiff) then begin
          Result:=false;
          Exit;
        end;
      end;
    end;
    Result:=true;
  end;

initialization
  isnotNTGot:=TRUE;
end.

