{$I+}
unit main;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ComCtrls, StdCtrls, ExtCtrls, BrowseDir, BrowseFile;

type
  TMainForm = class(TForm)
    MainPC: TPageControl;
    PackTS: TTabSheet;
    UnpackTS: TTabSheet;
    PackBD: TBrowseDir;
    PackMemo: TMemo;
    MakeBtn: TButton;
    CopyrightL: TLabel;
    UnpackBF: TBrowseFile;
    OutDirBD: TBrowseDir;
    UnpackBtn: TButton;
    UnpackMemo: TMemo;
    SaveCatD: TSaveDialog;
    PackTypeRG: TRadioGroup;
    UnpackTypeRG: TRadioGroup;
    procedure FormCreate(Sender: TObject);
    procedure MakeBtnClick(Sender: TObject);
    procedure UnpackBtnClick(Sender: TObject);
    procedure UnpackBFSubEditChange(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  MainForm: TMainForm;

implementation

uses Littles;

{$R *.dfm}

const
  type0names : array[0..2] of string = ('INTRO.CAT', 'SOUND1.CAT', 'SOUND2.CAT');
  type1names : array[0..3] of string = ('ADLIB.CAT', 'AINTRO.CAT', 'RINTRO.CAT', 'ROLAND.CAT');
  type2names : array[0..4] of string = ('SAMPLE.CAT', 'SAMPLE2.CAT', 'SAMPLE3.CAT', 'GM.CAT', 'DRIVERS.CAT');

var
  typenames : array of array of string;

procedure TMainForm.FormCreate(Sender: TObject);
  procedure AssignArray(i: integer; const strings: array of string);
    var
      j : integer;
    begin
      SetLength(typenames[i],Length(strings));
      for j:=low(strings) to high(strings) do typenames[i][j]:=strings[j];
    end;
  begin
    FormToCenter(self);
    SetLength(typenames,3);
    AssignArray(0,type0names);
    AssignArray(1,type1names);
    AssignArray(2,type2names);
  end;

procedure TMainForm.MakeBtnClick(Sender: TObject);
  type
    PSearchRec = ^TSearchRec;
  var
    dsn : string;
    files : FileSearches;
    sortedfiles : TList;
    i,j : longint;
    nextoffset,len : DWORD;
    table : array of record
      offset,size : DWORD;
    end;
    names : array of ShortString;
    name : PString;
    fs,fd : file;
    buf : array of char;
  function CompareNames(Item1,Item2 : Pointer) : Integer;
    begin
      Result:=CompareText(PSearchRec(Item1)^.Name,PSearchRec(Item2)^.Name);
    end;
  begin
    MakeBtn.Enabled:=false; Application.ProcessMessages; MakeBtn.Enabled:=true;
    dsn:=PackBD.SubEdit.Text; if copy(dsn,length(dsn),1)='\' then dsn:=copy(dsn,1,length(dsn)-1);
    if not DirectoryExists(dsn) then begin
      MessageDlg('The specified source directory does not exist!',mtError,[mbOK],0);
      Exit;
    end;
    try
      FindFiles(dsn+'\*.*',faFile,files);
      if (length(files)=0) then begin
        MessageDlg('The specified source directory does not contain any file!',mtError,[mbOK],0);
        Exit;
      end;
      if not SaveCatD.Execute then Exit;
      sortedfiles:=TList.Create;
      for i:=low(files) to high(files) do sortedfiles.Add(@files[i]);
      sortedfiles.Sort(@CompareNames);
      SetLength(table,length(files));
      SetLength(names,length(files));
      nextoffset:=length(files)*2*sizeof(DWORD);
      for i:=low(files) to high(files) do begin
        table[i].offset:=nextoffset;
        table[i].size:=DWORD(PSearchRec(sortedfiles.Items[i])^.Size);
        if PackTypeRG.ItemIndex=1 then j:=0 else begin
          name:=addr(PSearchRec(sortedfiles.Items[i])^.Name);
          j:=pos('_',name^);
          if j=0 then names[i]:='' else begin
            names[i]:=copy(WithoutExt(name^),j+1,MaxInt);
            inc(names[i][0]);
            names[i][ord(names[i][0])]:=#0;
            if PackTypeRG.ItemIndex=0 then inc(table[i].size,ord(names[i][0]));
          end;
        end;
        if PackTypeRG.ItemIndex=0 then begin
          if (table[i].size>0) or (j>0) then inc(table[i].size,1);
          inc(nextoffset,table[i].size)
        end else begin
          if PackTypeRG.ItemIndex=2 then begin
            if (table[i].size>0) or (j>0) then inc(nextoffset,1);
            inc(nextoffset,ord(names[i][0]));
          end;
          inc(nextoffset,table[i].size);
        end;
      end;
      try
        AssignFile(fd,SaveCatD.FileName); Rewrite(fd,1);
        try
          BlockWrite(fd,table[0],sizeof(table[0])*length(table));
          for i:=low(files) to high(files) do begin
            len:=table[i].size;
            if PackTypeRG.ItemIndex<>1 then begin
              if (ord(names[i][0])>0) or (len>0) then BlockWrite(fd,names[i][0],1+ord(names[i][0]));
              if (PackTypeRG.ItemIndex=0) and (len>0) then dec(len,1+ord(names[i][0]));
            end;
            SetLength(buf,len);
            AssignFile(fs,dsn+'\'+PSearchRec(sortedfiles.Items[i])^.Name); Reset(fs,1);
            try
              if len>0 then begin
                BlockRead(fs,buf[0],len);
                BlockWrite(fd,buf[0],len);
              end;
            finally
              CloseFile(fs);
            end;
          end;
        finally
          CloseFile(fd);
        end;
      finally
        sortedfiles.Destroy;
      end;
    except
      on e:Exception do MessageDlg(e.Message,mtError,[mbOK],0);
    end;
  end;

procedure TMainForm.UnpackBtnClick(Sender: TObject);
  var
    fsn,ddn,fdn,ext,t : string;
    fs,fd : file;
    amount,i,len : DWORD;
    offsets,lengths : array of DWORD;
    name : array[byte] of char;
    namesize : byte;
    buf : array of char;
  begin
    UnpackBtn.Enabled:=false; Application.ProcessMessages; UnpackBtn.Enabled:=true;
    fsn:=UnpackBF.SubEdit.Text;
    ddn:=OutDirBD.SubEdit.Text; if copy(ddn,length(ddn),1)='\' then ddn:=copy(ddn,1,length(ddn)-1);
    if not FileExists(fsn) then begin
      MessageDlg('The specified CAT file does not exist!',mtError,[mbOK],0);
      Exit;
    end;
    if not DirectoryExists(ddn) then begin
      MessageDlg('The specified destination directory does not exist!',mtError,[mbOK],0);
      Exit;
    end;
    try
      AssignFile(fs,fsn); Reset(fs,1);
      try
        BlockRead(fs,amount,sizeof(amount));
        amount:=amount div (2*sizeof(amount));
        Seek(fs,0);
        SetLength(offsets,amount);
        SetLength(lengths,amount);
        for i:=0 to amount-1 do begin
          BlockRead(fs,offsets[i],sizeof(offsets[i]));
          BlockRead(fs,lengths[i],sizeof(lengths[i]));
        end;
        for i:=0 to amount-1 do begin
          len:=lengths[i];
          fdn:=ddn+'\'+IntToStr2(i,4);
          ext:='.raw';
          if len>0 then begin
            Seek(fs,offsets[i]);
            if UnpackTypeRG.ItemIndex<>1 then begin
              BlockRead(fs,namesize,sizeof(namesize));
              if namesize>0 then BlockRead(fs,name[0],namesize);
              if UnpackTypeRG.ItemIndex=0 then dec(len,1+namesize);
            end;
            SetLength(buf,len);
            BlockRead(fs,buf[0],len);
            if UnpackTypeRG.ItemIndex<>1 then
              if namesize>0 then fdn:=fdn+'_'+StrPas(name);
            if (len>=4) and (buf[0]='R') and (buf[1]='I') and (buf[2]='F') and (buf[3]='F') then ext:='.wav';
            if UnpackTypeRG.ItemIndex<>1 then
              if namesize>0 then begin
                t:=ExtractFileExt(StrPas(name));
                if t<>'' then ext:=t;
              end;
          end;
          fdn:=fdn+ext;
          AssignFile(fd,fdn); Rewrite(fd,1);
          try
            if len>0 then BlockWrite(fd,buf[0],len);
          finally
            CloseFile(fd);
          end;
        end;
      finally
        CloseFile(fs);
      end;
    except
      on e:Exception do MessageDlg(e.Message,mtError,[mbOK],0);
    end;
  end;

procedure TMainForm.UnpackBFSubEditChange(Sender: TObject);
  var
    name : string;
    i,j : integer;
  begin
    name:=UpperCase(OName(UnpackBF.SubEdit.Text));
    for i:=low(typenames) to high(typenames) do
      for j:=low(typenames[i]) to high(typenames[i]) do
        if name=typenames[i][j] then begin
          UnpackTypeRG.ItemIndex:=i;
          Exit;
        end;
  end;

end.
